package eu.jonahbauer.raytracing.render.renderer.pdf;

import eu.jonahbauer.raytracing.math.Vec3;
import eu.jonahbauer.raytracing.scene.Target;
import org.jetbrains.annotations.NotNull;

import java.util.List;
import java.util.Objects;
import java.util.random.RandomGenerator;

/**
 * A probability density function targeting a target.
 */
public record TargetingProbabilityDensityFunction(@NotNull Vec3 origin, @NotNull List<@NotNull Target> targets) implements ProbabilityDensityFunction {
    public TargetingProbabilityDensityFunction {
        Objects.requireNonNull(origin, "origin");
        targets = List.copyOf(targets);
    }

    @Override
    public double value(@NotNull Vec3 direction) {
        var weight = 1d / targets.size();
        var sum = 0.0;

        for (var target : targets) {
            sum += weight * target.getProbabilityDensity(origin, direction);
        }

        return sum;
    }

    @Override
    public @NotNull Vec3 generate(@NotNull RandomGenerator random) {
        return targets.get(random.nextInt(targets.size())).getTargetingDirection(origin, random);
    }
}
